@@ -8,6 +8,8 @@
 # libradosstriper headers, boost headers, etc. - are already installed.
 #
 
+source $(dirname $0)/../ceph-helpers-root.sh
+
 trap cleanup EXIT
 
 SOURCES="hello_radosstriper.cc
@@ -56,6 +58,14 @@ function run_binaries () {
 }
 
 pushd $DESTDIR
+case $(distro_id) in
+    centos|fedora|rhel|opensuse*|suse|sles)
+        install gcc-c++ make librados-devel;;
+    ubuntu|debian|devuan)
+        install g++ make librados-dev;;
+    *)
+        echo "$(distro_id) is unknown, $@ will have to be installed manually."
+esac
 get_sources
 check_sources
 make all-system
