@@ -452,6 +452,9 @@ class MDSRank {
 
     bool evict_client(int64_t session_id, bool wait, bool blacklist,
                       std::stringstream& ss, Context *on_killed=nullptr);
+    int config_client(int64_t session_id, bool remove,
+		      const std::string& option, const std::string& value,
+		      std::stringstream& ss);
 
   protected:
     void dump_clientreplay_status(Formatter *f) const;
