@@ -1326,9 +1326,10 @@ TEST(CrushWrapper, try_remap_rule) {
     vector<int> orig = { 0, 3, 9 };
     set<int> overfull = { 3 };
     vector<int> underfull = { 0, 2, 5, 8, 11 };
+    vector<int> more_underfull = {};
     vector<int> out;
     int r = c.try_remap_rule(g_ceph_context, rule, 3,
-			      overfull, underfull,
+			      overfull, underfull, more_underfull,
 			      orig, &out);
     cout << orig << " -> r = " << (int)r << " out " << out << std::endl;
     ASSERT_EQ(r, 0);
@@ -1342,7 +1343,7 @@ TEST(CrushWrapper, try_remap_rule) {
     orig = {1, 3, 9};
 
     r = c.try_remap_rule(g_ceph_context, rule, 3,
-			 overfull, underfull,
+			 overfull, underfull, more_underfull,
 			 orig, &out);
     cout << orig << " -> r = " << (int)r << " out " << out << std::endl;
     ASSERT_EQ(r, 0);
@@ -1350,6 +1351,21 @@ TEST(CrushWrapper, try_remap_rule) {
     ASSERT_EQ(1, out[0]);
     ASSERT_EQ(0, out[1]);
     ASSERT_EQ(9, out[2]);
+    //
+    // Check that more_underfull is used when underfull runs out
+    orig = { 0, 3, 9 };
+    overfull = { 3, 9 };
+    underfull = { 2 };
+    more_underfull = { 5, 8, 11 };
+    r = c.try_remap_rule(g_ceph_context, rule, 3,
+			      overfull, underfull, more_underfull,
+			      orig, &out);
+    cout << orig << " -> r = " << (int)r << " out " << out << std::endl;
+    ASSERT_EQ(r, 0);
+    ASSERT_EQ(3u, out.size());
+    ASSERT_EQ(0, out[0]);
+    ASSERT_EQ(2, out[1]);
+    ASSERT_EQ(5, out[2]);
   }
 
   // chooseleaf
@@ -1363,9 +1379,10 @@ TEST(CrushWrapper, try_remap_rule) {
     vector<int> orig = { 0, 3, 9 };
     set<int> overfull = { 3 };
     vector<int> underfull = { 0, 2, 5, 8, 11 };
+    vector<int> more_underfull = { };
     vector<int> out;
     int r = c.try_remap_rule(g_ceph_context, rule, 3,
-			      overfull, underfull,
+			      overfull, underfull, more_underfull,
 			      orig, &out);
     cout << orig << " -> r = " << (int)r << " out " << out << std::endl;
     ASSERT_EQ(r, 0);
@@ -1389,9 +1406,10 @@ TEST(CrushWrapper, try_remap_rule) {
     vector<int> orig = { 0, 3, 16, 12 };
     set<int> overfull = { 3, 12 };
     vector<int> underfull = { 6, 7, 9, 3, 0, 1, 15, 16, 13, 2, 5, 8, 11 };
+    vector<int> more_underfull = { };
     vector<int> out;
     int r = c.try_remap_rule(g_ceph_context, rule, 3,
-			      overfull, underfull,
+			      overfull, underfull, more_underfull,
 			      orig, &out);
     cout << orig << " -> r = " << (int)r << " out " << out << std::endl;
     ASSERT_EQ(r, 0);
@@ -1404,7 +1422,7 @@ TEST(CrushWrapper, try_remap_rule) {
     orig.pop_back();
     out.clear();
     r = c.try_remap_rule(g_ceph_context, rule, 3,
-			 overfull, underfull,
+			 overfull, underfull, more_underfull,
 			 orig, &out);
     cout << orig << " -> r = " << (int)r << " out " << out << std::endl;
     ASSERT_EQ(r, 0);
